/*
 * Copyright Redis Ltd. 2016 - present
 * Licensed under your choice of the Redis Source Available License 2.0 (RSALv2) or
 * the Server Side Public License v1 (SSPLv1).
 */

/* This file is automatically generated by Lemon from input grammar
** source file "parser.y". */
/*
** 2000-05-29
**
** The author disclaims copyright to this source code.  In place of
** a legal notice, here is a blessing:
**
**    May you do good and not evil.
**    May you find forgiveness for yourself and forgive others.
**    May you share freely, never taking more than you give.
**
*************************************************************************
** Driver template for the LEMON parser generator.
**
** The "lemon" program processes an LALR(1) input grammar file, then uses
** this template to construct a parser.  The "lemon" program inserts text
** at each "%%" line.  Also, any "P-a-r-s-e" identifer prefix (without the
** interstitial "-" characters) contained in this template is changed into
** the value of the %name directive from the grammar.  Otherwise, the content
** of this template is copied straight through into the generate parser
** source file.
**
** The following is the concatenation of all %include directives from the
** input grammar file:
*/
/************ Begin %include sections from the grammar ************************/
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <assert.h>

#include "../parse.h"
#include "util/arr.h"
#include "rmutil/vector.h"
#include "query_node.h"

// strndup + lowercase in one pass!
static char *strdupcase(const char *s, size_t len) {
  char *ret = rm_strndup(s, len);
  char *dst = ret;
  char *src = dst;
  while (*src) {
      // unescape 
      if (*src == '\\' && (ispunct(*(src+1)) || isspace(*(src+1)))) {
          ++src;
          continue;
      }
      *dst = tolower(*src);
      ++dst;
      ++src;

  }
  *dst = '\0';
  
  return ret;
}

// unescape a string (non null terminated) and return the new length (may be shorter than the original. This manipulates the string itself 
static size_t unescapen(char *s, size_t sz) {
  
  char *dst = s;
  char *src = dst;
  char *end = s + sz;
  while (src < end) {
      // unescape 
      if (*src == '\\' && src + 1 < end &&
         (ispunct(*(src+1)) || isspace(*(src+1)))) {
          ++src;
          continue;
      }
      *dst++ = *src++;
  }
 
  return (size_t)(dst - s);
}

#define NODENN_BOTH_VALID 0
#define NODENN_BOTH_INVALID -1
#define NODENN_ONE_NULL 1 
// Returns:
// 0 if a && b
// -1 if !a && !b
// 1 if a ^ b (i.e. !(a&&b||!a||!b)). The result is stored in `out` 
static int one_not_null(void *a, void *b, void *out) {
    if (a && b) {
        return NODENN_BOTH_VALID;
    } else if (a == NULL && b == NULL) {
        return NODENN_BOTH_INVALID;
    } if (a) {
        *(void **)out = a;
        return NODENN_ONE_NULL;
    } else {
        *(void **)out = b;
        return NODENN_ONE_NULL;
    }
}
   
/**************** End of %include directives **********************************/
/* These constants specify the various numeric values for terminal symbols.
***************** Begin token definitions *************************************/
#ifndef LOWEST
#define LOWEST                          1
#define TILDE                           2
#define TAGLIST                         3
#define QUOTE                           4
#define COLON                           5
#define MINUS                           6
#define NUMBER                          7
#define STOPWORD                        8
#define TERMLIST                        9
#define TERM                           10
#define PREFIX                         11
#define SUFFIX                         12
#define CONTAINS                       13
#define PERCENT                        14
#define ATTRIBUTE                      15
#define LP                             16
#define RP                             17
#define MODIFIER                       18
#define AND                            19
#define OR                             20
#define ORX                            21
#define ARROW                          22
#define STAR                           23
#define SEMICOLON                      24
#define LB                             25
#define RB                             26
#define LSQB                           27
#define RSQB                           28
#endif
/**************** End token definitions ***************************************/

/* The next sections is a series of control #defines.
** various aspects of the generated parser.
**    YYCODETYPE         is the data type used to store the integer codes
**                       that represent terminal and non-terminal symbols.
**                       "unsigned char" is used if there are fewer than
**                       256 symbols.  Larger types otherwise.
**    YYNOCODE           is a number of type YYCODETYPE that is not used for
**                       any terminal or nonterminal symbol.
**    YYFALLBACK         If defined, this indicates that one or more tokens
**                       (also known as: "terminal symbols") have fall-back
**                       values which should be used if the original symbol
**                       would not parse.  This permits keywords to sometimes
**                       be used as identifiers, for example.
**    YYACTIONTYPE       is the data type used for "action codes" - numbers
**                       that indicate what to do in response to the next
**                       token.
**    RSQueryParser_v1_TOKENTYPE     is the data type used for minor type for terminal
**                       symbols.  Background: A "minor type" is a semantic
**                       value associated with a terminal or non-terminal
**                       symbols.  For example, for an "ID" terminal symbol,
**                       the minor type might be the name of the identifier.
**                       Each non-terminal can have a different minor type.
**                       Terminal symbols all have the same minor type, though.
**                       This macros defines the minor type for terminal 
**                       symbols.
**    YYMINORTYPE        is the data type used for all minor types.
**                       This is typically a union of many types, one of
**                       which is RSQueryParser_v1_TOKENTYPE.  The entry in the union
**                       for terminal symbols is called "yy0".
**    YYSTACKDEPTH       is the maximum depth of the parser's stack.  If
**                       zero the stack is dynamically sized using realloc()
**    RSQueryParser_v1_ARG_SDECL     A static variable declaration for the %extra_argument
**    RSQueryParser_v1_ARG_PDECL     A parameter declaration for the %extra_argument
**    RSQueryParser_v1_ARG_PARAM     Code to pass %extra_argument as a subroutine parameter
**    RSQueryParser_v1_ARG_STORE     Code to store %extra_argument into yypParser
**    RSQueryParser_v1_ARG_FETCH     Code to extract %extra_argument from yypParser
**    RSQueryParser_v1_CTX_*         As RSQueryParser_v1_ARG_ except for %extra_context
**    YYERRORSYMBOL      is the code number of the error symbol.  If not
**                       defined, then do no error processing.
**    YYNSTATE           the combined number of states.
**    YYNRULE            the number of rules in the grammar
**    YYNTOKEN           Number of terminal symbols
**    YY_MAX_SHIFT       Maximum value for shift actions
**    YY_MIN_SHIFTREDUCE Minimum value for shift-reduce actions
**    YY_MAX_SHIFTREDUCE Maximum value for shift-reduce actions
**    YY_ERROR_ACTION    The yy_action[] code for syntax error
**    YY_ACCEPT_ACTION   The yy_action[] code for accept
**    YY_NO_ACTION       The yy_action[] code for no-op
**    YY_MIN_REDUCE      Minimum value for reduce actions
**    YY_MAX_REDUCE      Maximum value for reduce actions
*/
#ifndef INTERFACE
# define INTERFACE 1
#endif
/************* Begin control #defines *****************************************/
#define YYCODETYPE unsigned char
#define YYNOCODE 44
#define YYACTIONTYPE unsigned char
#define RSQueryParser_v1_TOKENTYPE QueryToken
typedef union {
  int yyinit;
  RSQueryParser_v1_TOKENTYPE yy0;
  QueryAttribute * yy1;
  Vector* yy42;
  RangeNumber yy47;
  QueryParam * yy62;
  QueryNode * yy75;
  QueryAttribute yy87;
} YYMINORTYPE;
#ifndef YYSTACKDEPTH
#define YYSTACKDEPTH 100
#endif
#define RSQueryParser_v1_ARG_SDECL  QueryParseCtx *ctx ;
#define RSQueryParser_v1_ARG_PDECL , QueryParseCtx *ctx 
#define RSQueryParser_v1_ARG_PARAM ,ctx 
#define RSQueryParser_v1_ARG_FETCH  QueryParseCtx *ctx =yypParser->ctx ;
#define RSQueryParser_v1_ARG_STORE yypParser->ctx =ctx ;
#define RSQueryParser_v1_CTX_SDECL
#define RSQueryParser_v1_CTX_PDECL
#define RSQueryParser_v1_CTX_PARAM
#define RSQueryParser_v1_CTX_FETCH
#define RSQueryParser_v1_CTX_STORE
#define YYNSTATE             62
#define YYNRULE              58
#define YYNRULE_WITH_ACTION  58
#define YYNTOKEN             29
#define YY_MAX_SHIFT         61
#define YY_MIN_SHIFTREDUCE   101
#define YY_MAX_SHIFTREDUCE   158
#define YY_ERROR_ACTION      159
#define YY_ACCEPT_ACTION     160
#define YY_NO_ACTION         161
#define YY_MIN_REDUCE        162
#define YY_MAX_REDUCE        219
/************* End control #defines *******************************************/
#define YY_NLOOKAHEAD ((int)(sizeof(yy_lookahead)/sizeof(yy_lookahead[0])))

/* Define the yytestcase() macro to be a no-op if is not already defined
** otherwise.
**
** Applications can choose to define yytestcase() in the %include section
** to a macro that can assist in verifying code coverage.  For production
** code the yytestcase() macro should be turned off.  But it is useful
** for testing.
*/
#ifndef yytestcase
# define yytestcase(X)
#endif


/* Next are the tables used to determine what action to take based on the
** current state and lookahead token.  These tables are used to implement
** functions that take a state number and lookahead value and return an
** action integer.  
**
** Suppose the action integer is N.  Then the action is determined as
** follows
**
**   0 <= N <= YY_MAX_SHIFT             Shift N.  That is, push the lookahead
**                                      token onto the stack and goto state N.
**
**   N between YY_MIN_SHIFTREDUCE       Shift to an arbitrary state then
**     and YY_MAX_SHIFTREDUCE           reduce by rule N-YY_MIN_SHIFTREDUCE.
**
**   N == YY_ERROR_ACTION               A syntax error has occurred.
**
**   N == YY_ACCEPT_ACTION              The parser accepts its input.
**
**   N == YY_NO_ACTION                  No such action.  Denotes unused
**                                      slots in the yy_action[] table.
**
**   N between YY_MIN_REDUCE            Reduce by rule N-YY_MIN_REDUCE
**     and YY_MAX_REDUCE
**
** The action table is constructed as a single large table named yy_action[].
** Given state S and lookahead X, the action is computed as either:
**
**    (A)   N = yy_action[ yy_shift_ofst[S] + X ]
**    (B)   N = yy_default[S]
**
** The (A) formula is preferred.  The B formula is used instead if
** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X.
**
** The formulas above are for computing the action when the lookahead is
** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
** a reduce action) then the yy_reduce_ofst[] array is used in place of
** the yy_shift_ofst[] array.
**
** The following are the tables generated in this section:
**
**  yy_action[]        A single table containing all actions.
**  yy_lookahead[]     A table containing the lookahead for each entry in
**                     yy_action.  Used to detect hash collisions.
**  yy_shift_ofst[]    For each state, the offset into yy_action for
**                     shifting terminals.
**  yy_reduce_ofst[]   For each state, the offset into yy_action for
**                     shifting non-terminals after a reduce.
**  yy_default[]       Default action for each state.
**
*********** Begin parsing tables **********************************************/
#define YY_ACTTAB_COUNT (276)
static const YYACTIONTYPE yy_action[] = {
 /*     0 */   173,   42,    5,   47,   19,   24,    6,  158,  122,   58,
 /*    10 */   157,  128,  129,  130,   23,   32,    7,  110,  137,  162,
 /*    20 */     9,    5,   61,   19,   61,    6,  158,  122,   18,  157,
 /*    30 */   128,  129,  130,   23,  149,    7,  163,  137,    5,    9,
 /*    40 */    19,   61,    6,  158,  122,   60,  157,  128,  129,  130,
 /*    50 */    23,    9,    7,   61,  137,   25,  154,  117,   59,   45,
 /*    60 */   158,  125,    5,  157,   19,   26,    6,  158,  122,  185,
 /*    70 */   157,  128,  129,  130,   23,   56,    7,  151,  137,  158,
 /*    80 */    50,  158,  157,  217,  157,   17,   31,   28,  158,   48,
 /*    90 */    19,  157,    6,  158,  122,   22,  157,  128,  129,  130,
 /*   100 */    23,   40,    7,  116,  137,    5,    9,   19,   61,    6,
 /*   110 */   158,  122,  216,  157,  128,  129,  130,   23,   21,    7,
 /*   120 */   157,  137,  158,  122,   27,  157,  128,  129,  130,   23,
 /*   130 */   184,    7,  199,  137,  200,    9,   13,   61,  172,  181,
 /*   140 */    39,  166,  174,   41,  213,   43,  164,  211,  153,   44,
 /*   150 */    37,    3,   46,    8,  181,   39,  166,   25,  154,    1,
 /*   160 */    43,  204,   29,  160,   44,   37,   14,   26,   36,  181,
 /*   170 */    39,  166,   34,    4,   33,   43,  181,   39,  166,   44,
 /*   180 */    37,  134,   43,  135,   49,   11,   44,   37,  181,   39,
 /*   190 */   166,  136,  161,   51,   43,  208,   30,   52,   44,   37,
 /*   200 */     2,  133,   54,  181,   39,  166,   35,   12,   55,   43,
 /*   210 */   181,   39,  166,   44,   37,  132,   43,   57,  131,   15,
 /*   220 */    44,   37,  181,   39,  166,   38,  161,   20,   43,  161,
 /*   230 */   161,  161,   44,   37,   16,  161,  161,  181,   39,  166,
 /*   240 */   161,  158,  125,   43,  157,  161,  161,   44,   37,  161,
 /*   250 */   161,  158,  142,  161,  157,  128,  129,  130,  161,  158,
 /*   260 */   146,  161,  157,  128,  129,  130,  161,  158,   53,  118,
 /*   270 */   157,  161,  158,  161,  161,  157,
};
static const YYCODETYPE yy_lookahead[] = {
 /*     0 */    30,   31,    2,   39,    4,   33,    6,    7,    8,   43,
 /*    10 */    10,   11,   12,   13,   14,   43,   16,   17,   18,    0,
 /*    20 */    20,    2,   22,    4,   22,    6,    7,    8,   20,   10,
 /*    30 */    11,   12,   13,   14,   26,   16,    0,   18,    2,   20,
 /*    40 */     4,   22,    6,    7,    8,   15,   10,   11,   12,   13,
 /*    50 */    14,   20,   16,   22,   18,    6,    7,    4,   43,   23,
 /*    60 */     7,    8,    2,   10,    4,   16,    6,    7,    8,   43,
 /*    70 */    10,   11,   12,   13,   14,   43,   16,   28,   18,    7,
 /*    80 */     8,    7,   10,   39,   10,   25,   14,   27,    7,    8,
 /*    90 */     4,   10,    6,    7,    8,   14,   10,   11,   12,   13,
 /*   100 */    14,   24,   16,   26,   18,    2,   20,    4,   22,    6,
 /*   110 */     7,    8,   39,   10,   11,   12,   13,   14,   39,   16,
 /*   120 */    10,   18,    7,    8,   39,   10,   11,   12,   13,   14,
 /*   130 */    43,   16,   43,   18,   43,   20,   29,   22,   43,   32,
 /*   140 */    33,   34,   30,   36,   37,   38,    0,   40,   28,   42,
 /*   150 */    43,   29,   10,    5,   32,   33,   34,    6,    7,    5,
 /*   160 */    38,   32,   33,   41,   42,   43,   29,   16,   20,   32,
 /*   170 */    33,   34,   43,   29,   20,   38,   32,   33,   34,   42,
 /*   180 */    43,   14,   38,   14,   14,   29,   42,   43,   32,   33,
 /*   190 */    34,   14,   44,   14,   38,   32,   33,   14,   42,   43,
 /*   200 */    29,   14,   14,   32,   33,   34,   43,   29,   14,   38,
 /*   210 */    32,   33,   34,   42,   43,   14,   38,   14,   14,   29,
 /*   220 */    42,   43,   32,   33,   34,    5,   44,   25,   38,   44,
 /*   230 */    44,   44,   42,   43,   29,   44,   44,   32,   33,   34,
 /*   240 */    44,    7,    8,   38,   10,   44,   44,   42,   43,   44,
 /*   250 */    44,    7,    8,   44,   10,   11,   12,   13,   44,    7,
 /*   260 */     8,   44,   10,   11,   12,   13,   44,    7,    8,    4,
 /*   270 */    10,   44,    7,   44,   44,   10,   44,   44,   44,   44,
 /*   280 */    44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
 /*   290 */    44,   44,   44,   44,   44,   44,   44,   44,   44,   29,
 /*   300 */    29,   29,   29,   29,   29,
};
#define YY_SHIFT_COUNT    (61)
#define YY_SHIFT_MIN      (0)
#define YY_SHIFT_MAX      (265)
static const unsigned short int yy_shift_ofst[] = {
 /*     0 */    36,   60,    0,   19,   86,  103,  103,  103,  103,  103,
 /*    10 */   103,  115,   31,   31,   31,    2,    2,  244,  252,   74,
 /*    20 */    30,   49,   72,   81,   53,  151,  151,  151,  151,  234,
 /*    30 */   234,  260,  265,   74,   74,   74,   74,   74,   74,  110,
 /*    40 */    30,    8,   77,  148,  154,  146,  120,  142,  167,  169,
 /*    50 */   170,  177,  179,  183,  187,  188,  194,  201,  203,  204,
 /*    60 */   220,  202,
};
#define YY_REDUCE_COUNT (40)
#define YY_REDUCE_MIN   (-36)
#define YY_REDUCE_MAX   (205)
static const short yy_reduce_ofst[] = {
 /*     0 */   122,  107,  137,  137,  137,  144,  156,  171,  178,  190,
 /*    10 */   205,  137,  137,  137,  137,  137,  137,  129,  163,  -28,
 /*    20 */   -30,  -36,  -34,   15,   26,   44,   73,   79,   85,   26,
 /*    30 */    26,   32,   87,   89,   87,   87,   91,   87,   95,   26,
 /*    40 */   112,
};
static const YYACTIONTYPE yy_default[] = {
 /*     0 */   159,  159,  159,  159,  188,  159,  159,  159,  159,  159,
 /*    10 */   159,  187,  170,  169,  165,  167,  168,  159,  159,  159,
 /*    20 */   176,  159,  159,  159,  159,  159,  159,  159,  159,  205,
 /*    30 */   209,  159,  159,  159,  202,  206,  159,  180,  159,  182,
 /*    40 */   175,  201,  159,  159,  159,  159,  159,  159,  159,  159,
 /*    50 */   159,  159,  159,  159,  159,  159,  159,  159,  159,  159,
 /*    60 */   159,  159,
};
/********** End of lemon-generated parsing tables *****************************/

/* The next table maps tokens (terminal symbols) into fallback tokens.  
** If a construct like the following:
** 
**      %fallback ID X Y Z.
**
** appears in the grammar, then ID becomes a fallback token for X, Y,
** and Z.  Whenever one of the tokens X, Y, or Z is input to the parser
** but it does not parse, the type of the token is changed to ID and
** the parse is retried before an error is thrown.
**
** This feature can be used, for example, to cause some keywords in a language
** to revert to identifiers if they keyword does not apply in the context where
** it appears.
*/
#ifdef YYFALLBACK
static const YYCODETYPE yyFallback[] = {
};
#endif /* YYFALLBACK */

/* The following structure represents a single element of the
** parser's stack.  Information stored includes:
**
**   +  The state number for the parser at this level of the stack.
**
**   +  The value of the token stored at this level of the stack.
**      (In other words, the "major" token.)
**
**   +  The semantic value stored at this level of the stack.  This is
**      the information used by the action routines in the grammar.
**      It is sometimes called the "minor" token.
**
** After the "shift" half of a SHIFTREDUCE action, the stateno field
** actually contains the reduce action for the second half of the
** SHIFTREDUCE.
*/
struct yyStackEntry {
  YYACTIONTYPE stateno;  /* The state-number, or reduce action in SHIFTREDUCE */
  YYCODETYPE major;      /* The major token value.  This is the code
                         ** number for the token at this stack level */
  YYMINORTYPE minor;     /* The user-supplied minor token value.  This
                         ** is the value of the token  */
};
typedef struct yyStackEntry yyStackEntry;

/* The state of the parser is completely contained in an instance of
** the following structure */
struct yyParser {
  yyStackEntry *yytos;          /* Pointer to top element of the stack */
#ifdef YYTRACKMAXSTACKDEPTH
  int yyhwm;                    /* High-water mark of the stack */
#endif
#ifndef YYNOERRORRECOVERY
  int yyerrcnt;                 /* Shifts left before out of the error */
#endif
  RSQueryParser_v1_ARG_SDECL                /* A place to hold %extra_argument */
  RSQueryParser_v1_CTX_SDECL                /* A place to hold %extra_context */
#if YYSTACKDEPTH<=0
  int yystksz;                  /* Current side of the stack */
  yyStackEntry *yystack;        /* The parser's stack */
  yyStackEntry yystk0;          /* First stack entry */
#else
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
  yyStackEntry *yystackEnd;            /* Last entry in the stack */
#endif
};
typedef struct yyParser yyParser;

#include <assert.h>
#ifndef NDEBUG
#include <stdio.h>
static FILE *yyTraceFILE = 0;
static char *yyTracePrompt = 0;
#endif /* NDEBUG */

#ifndef NDEBUG
/* 
** Turn parser tracing on by giving a stream to which to write the trace
** and a prompt to preface each trace message.  Tracing is turned off
** by making either argument NULL 
**
** Inputs:
** <ul>
** <li> A FILE* to which trace output should be written.
**      If NULL, then tracing is turned off.
** <li> A prefix string written at the beginning of every
**      line of trace output.  If NULL, then tracing is
**      turned off.
** </ul>
**
** Outputs:
** None.
*/
void RSQueryParser_v1_Trace(FILE *TraceFILE, char *zTracePrompt){
  yyTraceFILE = TraceFILE;
  yyTracePrompt = zTracePrompt;
  if( yyTraceFILE==0 ) yyTracePrompt = 0;
  else if( yyTracePrompt==0 ) yyTraceFILE = 0;
}
#endif /* NDEBUG */

#if defined(YYCOVERAGE) || !defined(NDEBUG)
/* For tracing shifts, the names of all terminals and nonterminals
** are required.  The following table supplies these names */
static const char *const yyTokenName[] = { 
  /*    0 */ "$",
  /*    1 */ "LOWEST",
  /*    2 */ "TILDE",
  /*    3 */ "TAGLIST",
  /*    4 */ "QUOTE",
  /*    5 */ "COLON",
  /*    6 */ "MINUS",
  /*    7 */ "NUMBER",
  /*    8 */ "STOPWORD",
  /*    9 */ "TERMLIST",
  /*   10 */ "TERM",
  /*   11 */ "PREFIX",
  /*   12 */ "SUFFIX",
  /*   13 */ "CONTAINS",
  /*   14 */ "PERCENT",
  /*   15 */ "ATTRIBUTE",
  /*   16 */ "LP",
  /*   17 */ "RP",
  /*   18 */ "MODIFIER",
  /*   19 */ "AND",
  /*   20 */ "OR",
  /*   21 */ "ORX",
  /*   22 */ "ARROW",
  /*   23 */ "STAR",
  /*   24 */ "SEMICOLON",
  /*   25 */ "LB",
  /*   26 */ "RB",
  /*   27 */ "LSQB",
  /*   28 */ "RSQB",
  /*   29 */ "expr",
  /*   30 */ "attribute",
  /*   31 */ "attribute_list",
  /*   32 */ "affix",
  /*   33 */ "termlist",
  /*   34 */ "union",
  /*   35 */ "fuzzy",
  /*   36 */ "tag_list",
  /*   37 */ "geo_filter",
  /*   38 */ "modifierlist",
  /*   39 */ "num",
  /*   40 */ "numeric_range",
  /*   41 */ "query",
  /*   42 */ "modifier",
  /*   43 */ "term",
};
#endif /* defined(YYCOVERAGE) || !defined(NDEBUG) */

#ifndef NDEBUG
/* For tracing reduce actions, the names of all rules are required.
*/
static const char *const yyRuleName[] = {
 /*   0 */ "query ::= expr",
 /*   1 */ "query ::=",
 /*   2 */ "query ::= STAR",
 /*   3 */ "expr ::= expr expr",
 /*   4 */ "expr ::= union",
 /*   5 */ "union ::= expr OR expr",
 /*   6 */ "union ::= union OR expr",
 /*   7 */ "expr ::= modifier COLON expr",
 /*   8 */ "expr ::= modifierlist COLON expr",
 /*   9 */ "expr ::= LP expr RP",
 /*  10 */ "attribute ::= ATTRIBUTE COLON term",
 /*  11 */ "attribute_list ::= attribute",
 /*  12 */ "attribute_list ::= attribute_list SEMICOLON attribute",
 /*  13 */ "attribute_list ::= attribute_list SEMICOLON",
 /*  14 */ "attribute_list ::=",
 /*  15 */ "expr ::= expr ARROW LB attribute_list RB",
 /*  16 */ "expr ::= QUOTE termlist QUOTE",
 /*  17 */ "expr ::= QUOTE term QUOTE",
 /*  18 */ "expr ::= term",
 /*  19 */ "expr ::= affix",
 /*  20 */ "expr ::= termlist",
 /*  21 */ "expr ::= STOPWORD",
 /*  22 */ "termlist ::= term term",
 /*  23 */ "termlist ::= termlist term",
 /*  24 */ "termlist ::= termlist STOPWORD",
 /*  25 */ "expr ::= MINUS expr",
 /*  26 */ "expr ::= TILDE expr",
 /*  27 */ "affix ::= PREFIX",
 /*  28 */ "affix ::= SUFFIX",
 /*  29 */ "affix ::= CONTAINS",
 /*  30 */ "expr ::= PERCENT term PERCENT",
 /*  31 */ "expr ::= PERCENT PERCENT term PERCENT PERCENT",
 /*  32 */ "expr ::= PERCENT PERCENT PERCENT term PERCENT PERCENT PERCENT",
 /*  33 */ "expr ::= PERCENT STOPWORD PERCENT",
 /*  34 */ "expr ::= PERCENT PERCENT STOPWORD PERCENT PERCENT",
 /*  35 */ "expr ::= PERCENT PERCENT PERCENT STOPWORD PERCENT PERCENT PERCENT",
 /*  36 */ "modifier ::= MODIFIER",
 /*  37 */ "modifierlist ::= modifier OR term",
 /*  38 */ "modifierlist ::= modifierlist OR term",
 /*  39 */ "expr ::= modifier COLON tag_list",
 /*  40 */ "tag_list ::= LB term",
 /*  41 */ "tag_list ::= LB STOPWORD",
 /*  42 */ "tag_list ::= LB affix",
 /*  43 */ "tag_list ::= LB termlist",
 /*  44 */ "tag_list ::= tag_list OR term",
 /*  45 */ "tag_list ::= tag_list OR STOPWORD",
 /*  46 */ "tag_list ::= tag_list OR affix",
 /*  47 */ "tag_list ::= tag_list OR termlist",
 /*  48 */ "tag_list ::= tag_list RB",
 /*  49 */ "expr ::= modifier COLON numeric_range",
 /*  50 */ "numeric_range ::= LSQB num num RSQB",
 /*  51 */ "expr ::= modifier COLON geo_filter",
 /*  52 */ "geo_filter ::= LSQB num num num TERM RSQB",
 /*  53 */ "num ::= NUMBER",
 /*  54 */ "num ::= LP num",
 /*  55 */ "num ::= MINUS num",
 /*  56 */ "term ::= TERM",
 /*  57 */ "term ::= NUMBER",
};
#endif /* NDEBUG */


#if YYSTACKDEPTH<=0
/*
** Try to increase the size of the parser stack.  Return the number
** of errors.  Return 0 on success.
*/
static int yyGrowStack(yyParser *p){
  int newSize;
  int idx;
  yyStackEntry *pNew;

  newSize = p->yystksz*2 + 100;
  idx = p->yytos ? (int)(p->yytos - p->yystack) : 0;
  if( p->yystack==&p->yystk0 ){
    pNew = malloc(newSize*sizeof(pNew[0]));
    if( pNew ) pNew[0] = p->yystk0;
  }else{
    pNew = realloc(p->yystack, newSize*sizeof(pNew[0]));
  }
  if( pNew ){
    p->yystack = pNew;
    p->yytos = &p->yystack[idx];
#ifndef NDEBUG
    if( yyTraceFILE ){
      fprintf(yyTraceFILE,"%sStack grows from %d to %d entries.\n",
              yyTracePrompt, p->yystksz, newSize);
    }
#endif
    p->yystksz = newSize;
  }
  return pNew==0; 
}
#endif

/* Datatype of the argument to the memory allocated passed as the
** second argument to RSQueryParser_v1_Alloc() below.  This can be changed by
** putting an appropriate #define in the %include section of the input
** grammar.
*/
#ifndef YYMALLOCARGTYPE
# define YYMALLOCARGTYPE size_t
#endif

/* Initialize a new parser that has already been allocated.
*/
void RSQueryParser_v1_Init(void *yypRawParser RSQueryParser_v1_CTX_PDECL){
  yyParser *yypParser = (yyParser*)yypRawParser;
  RSQueryParser_v1_CTX_STORE
#ifdef YYTRACKMAXSTACKDEPTH
  yypParser->yyhwm = 0;
#endif
#if YYSTACKDEPTH<=0
  yypParser->yytos = NULL;
  yypParser->yystack = NULL;
  yypParser->yystksz = 0;
  if( yyGrowStack(yypParser) ){
    yypParser->yystack = &yypParser->yystk0;
    yypParser->yystksz = 1;
  }
#endif
#ifndef YYNOERRORRECOVERY
  yypParser->yyerrcnt = -1;
#endif
  yypParser->yytos = yypParser->yystack;
  yypParser->yystack[0].stateno = 0;
  yypParser->yystack[0].major = 0;
#if YYSTACKDEPTH>0
  yypParser->yystackEnd = &yypParser->yystack[YYSTACKDEPTH-1];
#endif
}

#ifndef RSQueryParser_v1__ENGINEALWAYSONSTACK
/* 
** This function allocates a new parser.
** The only argument is a pointer to a function which works like
** malloc.
**
** Inputs:
** A pointer to the function used to allocate memory.
**
** Outputs:
** A pointer to a parser.  This pointer is used in subsequent calls
** to RSQueryParser_v1_ and RSQueryParser_v1_Free.
*/
void *RSQueryParser_v1_Alloc(void *(*mallocProc)(YYMALLOCARGTYPE) RSQueryParser_v1_CTX_PDECL){
  yyParser *yypParser;
  yypParser = (yyParser*)(*mallocProc)( (YYMALLOCARGTYPE)sizeof(yyParser) );
  if( yypParser ){
    RSQueryParser_v1_CTX_STORE
    RSQueryParser_v1_Init(yypParser RSQueryParser_v1_CTX_PARAM);
  }
  return (void*)yypParser;
}
#endif /* RSQueryParser_v1__ENGINEALWAYSONSTACK */


/* The following function deletes the "minor type" or semantic value
** associated with a symbol.  The symbol can be either a terminal
** or nonterminal. "yymajor" is the symbol code, and "yypminor" is
** a pointer to the value to be deleted.  The code used to do the 
** deletions is derived from the %destructor and/or %token_destructor
** directives of the input grammar.
*/
static void yy_destructor(
  yyParser *yypParser,    /* The parser */
  YYCODETYPE yymajor,     /* Type code for object to destroy */
  YYMINORTYPE *yypminor   /* The object to be destroyed */
){
  RSQueryParser_v1_ARG_FETCH
  RSQueryParser_v1_CTX_FETCH
  switch( yymajor ){
    /* Here is inserted the actions which take place when a
    ** terminal or non-terminal is destroyed.  This can happen
    ** when the symbol is popped from the stack during a
    ** reduce or during error processing or when a parser is 
    ** being destroyed before it is finished parsing.
    **
    ** Note: during a reduce, the only symbols destroyed are those
    ** which appear on the RHS of the rule, but which are *not* used
    ** inside the C code.
    */
/********* Begin destructor definitions ***************************************/
      /* Default NON-TERMINAL Destructor */
    case 39: /* num */
    case 41: /* query */
    case 42: /* modifier */
    case 43: /* term */
{
 
}
      break;
    case 29: /* expr */
    case 32: /* affix */
    case 33: /* termlist */
    case 34: /* union */
    case 35: /* fuzzy */
    case 36: /* tag_list */
{
 QueryNode_Free((yypminor->yy75)); 
}
      break;
    case 30: /* attribute */
{
 rm_free((char*)(yypminor->yy87).value); 
}
      break;
    case 31: /* attribute_list */
{
 array_free_ex((yypminor->yy1), rm_free((char*)((QueryAttribute*)ptr )->value)); 
}
      break;
    case 37: /* geo_filter */
    case 40: /* numeric_range */
{
 QueryParam_Free((yypminor->yy62)); 
}
      break;
    case 38: /* modifierlist */
{
 
    for (size_t i = 0; i < Vector_Size((yypminor->yy42)); i++) {
        char *s;
        Vector_Get((yypminor->yy42), i, &s);
        rm_free(s);
    }
    Vector_Free((yypminor->yy42)); 

}
      break;
/********* End destructor definitions *****************************************/
    default:  break;   /* If no destructor action specified: do nothing */
  }
}

/*
** Pop the parser's stack once.
**
** If there is a destructor routine associated with the token which
** is popped from the stack, then call it.
*/
static void yy_pop_parser_stack(yyParser *pParser){
  yyStackEntry *yytos;
  assert( pParser->yytos!=0 );
  assert( pParser->yytos > pParser->yystack );
  yytos = pParser->yytos--;
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sPopping %s\n",
      yyTracePrompt,
      yyTokenName[yytos->major]);
  }
#endif
  yy_destructor(pParser, yytos->major, &yytos->minor);
}

/*
** Clear all secondary memory allocations from the parser
*/
void RSQueryParser_v1_Finalize(void *p){
  yyParser *pParser = (yyParser*)p;
  while( pParser->yytos>pParser->yystack ) yy_pop_parser_stack(pParser);
#if YYSTACKDEPTH<=0
  if( pParser->yystack!=&pParser->yystk0 ) free(pParser->yystack);
#endif
}

#ifndef RSQueryParser_v1__ENGINEALWAYSONSTACK
/* 
** Deallocate and destroy a parser.  Destructors are called for
** all stack elements before shutting the parser down.
**
** If the YYPARSEFREENEVERNULL macro exists (for example because it
** is defined in a %include section of the input grammar) then it is
** assumed that the input pointer is never NULL.
*/
void RSQueryParser_v1_Free(
  void *p,                    /* The parser to be deleted */
  void (*freeProc)(void*)     /* Function used to reclaim memory */
){
#ifndef YYPARSEFREENEVERNULL
  if( p==0 ) return;
#endif
  RSQueryParser_v1_Finalize(p);
  (*freeProc)(p);
}
#endif /* RSQueryParser_v1__ENGINEALWAYSONSTACK */

/*
** Return the peak depth of the stack for a parser.
*/
#ifdef YYTRACKMAXSTACKDEPTH
int RSQueryParser_v1_StackPeak(void *p){
  yyParser *pParser = (yyParser*)p;
  return pParser->yyhwm;
}
#endif

/* This array of booleans keeps track of the parser statement
** coverage.  The element yycoverage[X][Y] is set when the parser
** is in state X and has a lookahead token Y.  In a well-tested
** systems, every element of this matrix should end up being set.
*/
#if defined(YYCOVERAGE)
static unsigned char yycoverage[YYNSTATE][YYNTOKEN];
#endif

/*
** Write into out a description of every state/lookahead combination that
**
**   (1)  has not been used by the parser, and
**   (2)  is not a syntax error.
**
** Return the number of missed state/lookahead combinations.
*/
#if defined(YYCOVERAGE)
int RSQueryParser_v1_Coverage(FILE *out){
  int stateno, iLookAhead, i;
  int nMissed = 0;
  for(stateno=0; stateno<YYNSTATE; stateno++){
    i = yy_shift_ofst[stateno];
    for(iLookAhead=0; iLookAhead<YYNTOKEN; iLookAhead++){
      if( yy_lookahead[i+iLookAhead]!=iLookAhead ) continue;
      if( yycoverage[stateno][iLookAhead]==0 ) nMissed++;
      if( out ){
        fprintf(out,"State %d lookahead %s %s\n", stateno,
                yyTokenName[iLookAhead],
                yycoverage[stateno][iLookAhead] ? "ok" : "missed");
      }
    }
  }
  return nMissed;
}
#endif

/*
** Find the appropriate action for a parser given the terminal
** look-ahead token iLookAhead.
*/
static YYACTIONTYPE yy_find_shift_action(
  YYCODETYPE iLookAhead,    /* The look-ahead token */
  YYACTIONTYPE stateno      /* Current state number */
){
  int i;

  if( stateno>YY_MAX_SHIFT ) return stateno;
  assert( stateno <= YY_SHIFT_COUNT );
#if defined(YYCOVERAGE)
  yycoverage[stateno][iLookAhead] = 1;
#endif
  do{
    i = yy_shift_ofst[stateno];
    assert( i>=0 );
    assert( i<=YY_ACTTAB_COUNT );
    assert( i+YYNTOKEN<=(int)YY_NLOOKAHEAD );
    assert( iLookAhead!=YYNOCODE );
    assert( iLookAhead < YYNTOKEN );
    i += iLookAhead;
    assert( i<(int)YY_NLOOKAHEAD );
    if( yy_lookahead[i]!=iLookAhead ){
#ifdef YYFALLBACK
      YYCODETYPE iFallback;            /* Fallback token */
      assert( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0]) );
      iFallback = yyFallback[iLookAhead];
      if( iFallback!=0 ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
             yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
        }
#endif
        assert( yyFallback[iFallback]==0 ); /* Fallback loop must terminate */
        iLookAhead = iFallback;
        continue;
      }
#endif
#ifdef YYWILDCARD
      {
        int j = i - iLookAhead + YYWILDCARD;
        assert( j<(int)(sizeof(yy_lookahead)/sizeof(yy_lookahead[0])) );
        if( yy_lookahead[j]==YYWILDCARD && iLookAhead>0 ){
#ifndef NDEBUG
          if( yyTraceFILE ){
            fprintf(yyTraceFILE, "%sWILDCARD %s => %s\n",
               yyTracePrompt, yyTokenName[iLookAhead],
               yyTokenName[YYWILDCARD]);
          }
#endif /* NDEBUG */
          return yy_action[j];
        }
      }
#endif /* YYWILDCARD */
      return yy_default[stateno];
    }else{
      assert( i>=0 && i<(int)(sizeof(yy_action)/sizeof(yy_action[0])) );
      return yy_action[i];
    }
  }while(1);
}

/*
** Find the appropriate action for a parser given the non-terminal
** look-ahead token iLookAhead.
*/
static YYACTIONTYPE yy_find_reduce_action(
  YYACTIONTYPE stateno,     /* Current state number */
  YYCODETYPE iLookAhead     /* The look-ahead token */
){
  int i;
#ifdef YYERRORSYMBOL
  if( stateno>YY_REDUCE_COUNT ){
    return yy_default[stateno];
  }
#else
  assert( stateno<=YY_REDUCE_COUNT );
#endif
  i = yy_reduce_ofst[stateno];
  assert( iLookAhead!=YYNOCODE );
  i += iLookAhead;
#ifdef YYERRORSYMBOL
  if( i<0 || i>=YY_ACTTAB_COUNT || yy_lookahead[i]!=iLookAhead ){
    return yy_default[stateno];
  }
#else
  assert( i>=0 && i<YY_ACTTAB_COUNT );
  assert( yy_lookahead[i]==iLookAhead );
#endif
  return yy_action[i];
}

/*
** The following routine is called if the stack overflows.
*/
static void yyStackOverflow(yyParser *yypParser){
   RSQueryParser_v1_ARG_FETCH
   RSQueryParser_v1_CTX_FETCH
#ifndef NDEBUG
   if( yyTraceFILE ){
     fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
   }
#endif
   while( yypParser->yytos>yypParser->yystack ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will execute if the parser
   ** stack every overflows */
/******** Begin %stack_overflow code ******************************************/
/******** End %stack_overflow code ********************************************/
   RSQueryParser_v1_ARG_STORE /* Suppress warning about unused %extra_argument var */
   RSQueryParser_v1_CTX_STORE
}

/*
** Print tracing information for a SHIFT action
*/
#ifndef NDEBUG
static void yyTraceShift(yyParser *yypParser, int yyNewState, const char *zTag){
  if( yyTraceFILE ){
    if( yyNewState<YYNSTATE ){
      fprintf(yyTraceFILE,"%s%s '%s', go to state %d\n",
         yyTracePrompt, zTag, yyTokenName[yypParser->yytos->major],
         yyNewState);
    }else{
      fprintf(yyTraceFILE,"%s%s '%s', pending reduce %d\n",
         yyTracePrompt, zTag, yyTokenName[yypParser->yytos->major],
         yyNewState - YY_MIN_REDUCE);
    }
  }
}
#else
# define yyTraceShift(X,Y,Z)
#endif

/*
** Perform a shift action.
*/
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  YYACTIONTYPE yyNewState,      /* The new state to shift in */
  YYCODETYPE yyMajor,           /* The major token to shift in */
  RSQueryParser_v1_TOKENTYPE yyMinor        /* The minor token to shift in */
){
  yyStackEntry *yytos;
  yypParser->yytos++;
#ifdef YYTRACKMAXSTACKDEPTH
  if( (int)(yypParser->yytos - yypParser->yystack)>yypParser->yyhwm ){
    yypParser->yyhwm++;
    assert( yypParser->yyhwm == (int)(yypParser->yytos - yypParser->yystack) );
  }
#endif
#if YYSTACKDEPTH>0 
  if( yypParser->yytos>yypParser->yystackEnd ){
    yypParser->yytos--;
    yyStackOverflow(yypParser);
    return;
  }
#else
  if( yypParser->yytos>=&yypParser->yystack[yypParser->yystksz] ){
    if( yyGrowStack(yypParser) ){
      yypParser->yytos--;
      yyStackOverflow(yypParser);
      return;
    }
  }
#endif
  if( yyNewState > YY_MAX_SHIFT ){
    yyNewState += YY_MIN_REDUCE - YY_MIN_SHIFTREDUCE;
  }
  yytos = yypParser->yytos;
  yytos->stateno = yyNewState;
  yytos->major = yyMajor;
  yytos->minor.yy0 = yyMinor;
  yyTraceShift(yypParser, yyNewState, "Shift");
}

/* For rule J, yyRuleInfoLhs[J] contains the symbol on the left-hand side
** of that rule */
static const YYCODETYPE yyRuleInfoLhs[] = {
    41,  /* (0) query ::= expr */
    41,  /* (1) query ::= */
    41,  /* (2) query ::= STAR */
    29,  /* (3) expr ::= expr expr */
    29,  /* (4) expr ::= union */
    34,  /* (5) union ::= expr OR expr */
    34,  /* (6) union ::= union OR expr */
    29,  /* (7) expr ::= modifier COLON expr */
    29,  /* (8) expr ::= modifierlist COLON expr */
    29,  /* (9) expr ::= LP expr RP */
    30,  /* (10) attribute ::= ATTRIBUTE COLON term */
    31,  /* (11) attribute_list ::= attribute */
    31,  /* (12) attribute_list ::= attribute_list SEMICOLON attribute */
    31,  /* (13) attribute_list ::= attribute_list SEMICOLON */
    31,  /* (14) attribute_list ::= */
    29,  /* (15) expr ::= expr ARROW LB attribute_list RB */
    29,  /* (16) expr ::= QUOTE termlist QUOTE */
    29,  /* (17) expr ::= QUOTE term QUOTE */
    29,  /* (18) expr ::= term */
    29,  /* (19) expr ::= affix */
    29,  /* (20) expr ::= termlist */
    29,  /* (21) expr ::= STOPWORD */
    33,  /* (22) termlist ::= term term */
    33,  /* (23) termlist ::= termlist term */
    33,  /* (24) termlist ::= termlist STOPWORD */
    29,  /* (25) expr ::= MINUS expr */
    29,  /* (26) expr ::= TILDE expr */
    32,  /* (27) affix ::= PREFIX */
    32,  /* (28) affix ::= SUFFIX */
    32,  /* (29) affix ::= CONTAINS */
    29,  /* (30) expr ::= PERCENT term PERCENT */
    29,  /* (31) expr ::= PERCENT PERCENT term PERCENT PERCENT */
    29,  /* (32) expr ::= PERCENT PERCENT PERCENT term PERCENT PERCENT PERCENT */
    29,  /* (33) expr ::= PERCENT STOPWORD PERCENT */
    29,  /* (34) expr ::= PERCENT PERCENT STOPWORD PERCENT PERCENT */
    29,  /* (35) expr ::= PERCENT PERCENT PERCENT STOPWORD PERCENT PERCENT PERCENT */
    42,  /* (36) modifier ::= MODIFIER */
    38,  /* (37) modifierlist ::= modifier OR term */
    38,  /* (38) modifierlist ::= modifierlist OR term */
    29,  /* (39) expr ::= modifier COLON tag_list */
    36,  /* (40) tag_list ::= LB term */
    36,  /* (41) tag_list ::= LB STOPWORD */
    36,  /* (42) tag_list ::= LB affix */
    36,  /* (43) tag_list ::= LB termlist */
    36,  /* (44) tag_list ::= tag_list OR term */
    36,  /* (45) tag_list ::= tag_list OR STOPWORD */
    36,  /* (46) tag_list ::= tag_list OR affix */
    36,  /* (47) tag_list ::= tag_list OR termlist */
    36,  /* (48) tag_list ::= tag_list RB */
    29,  /* (49) expr ::= modifier COLON numeric_range */
    40,  /* (50) numeric_range ::= LSQB num num RSQB */
    29,  /* (51) expr ::= modifier COLON geo_filter */
    37,  /* (52) geo_filter ::= LSQB num num num TERM RSQB */
    39,  /* (53) num ::= NUMBER */
    39,  /* (54) num ::= LP num */
    39,  /* (55) num ::= MINUS num */
    43,  /* (56) term ::= TERM */
    43,  /* (57) term ::= NUMBER */
};

/* For rule J, yyRuleInfoNRhs[J] contains the negative of the number
** of symbols on the right-hand side of that rule. */
static const signed char yyRuleInfoNRhs[] = {
   -1,  /* (0) query ::= expr */
    0,  /* (1) query ::= */
   -1,  /* (2) query ::= STAR */
   -2,  /* (3) expr ::= expr expr */
   -1,  /* (4) expr ::= union */
   -3,  /* (5) union ::= expr OR expr */
   -3,  /* (6) union ::= union OR expr */
   -3,  /* (7) expr ::= modifier COLON expr */
   -3,  /* (8) expr ::= modifierlist COLON expr */
   -3,  /* (9) expr ::= LP expr RP */
   -3,  /* (10) attribute ::= ATTRIBUTE COLON term */
   -1,  /* (11) attribute_list ::= attribute */
   -3,  /* (12) attribute_list ::= attribute_list SEMICOLON attribute */
   -2,  /* (13) attribute_list ::= attribute_list SEMICOLON */
    0,  /* (14) attribute_list ::= */
   -5,  /* (15) expr ::= expr ARROW LB attribute_list RB */
   -3,  /* (16) expr ::= QUOTE termlist QUOTE */
   -3,  /* (17) expr ::= QUOTE term QUOTE */
   -1,  /* (18) expr ::= term */
   -1,  /* (19) expr ::= affix */
   -1,  /* (20) expr ::= termlist */
   -1,  /* (21) expr ::= STOPWORD */
   -2,  /* (22) termlist ::= term term */
   -2,  /* (23) termlist ::= termlist term */
   -2,  /* (24) termlist ::= termlist STOPWORD */
   -2,  /* (25) expr ::= MINUS expr */
   -2,  /* (26) expr ::= TILDE expr */
   -1,  /* (27) affix ::= PREFIX */
   -1,  /* (28) affix ::= SUFFIX */
   -1,  /* (29) affix ::= CONTAINS */
   -3,  /* (30) expr ::= PERCENT term PERCENT */
   -5,  /* (31) expr ::= PERCENT PERCENT term PERCENT PERCENT */
   -7,  /* (32) expr ::= PERCENT PERCENT PERCENT term PERCENT PERCENT PERCENT */
   -3,  /* (33) expr ::= PERCENT STOPWORD PERCENT */
   -5,  /* (34) expr ::= PERCENT PERCENT STOPWORD PERCENT PERCENT */
   -7,  /* (35) expr ::= PERCENT PERCENT PERCENT STOPWORD PERCENT PERCENT PERCENT */
   -1,  /* (36) modifier ::= MODIFIER */
   -3,  /* (37) modifierlist ::= modifier OR term */
   -3,  /* (38) modifierlist ::= modifierlist OR term */
   -3,  /* (39) expr ::= modifier COLON tag_list */
   -2,  /* (40) tag_list ::= LB term */
   -2,  /* (41) tag_list ::= LB STOPWORD */
   -2,  /* (42) tag_list ::= LB affix */
   -2,  /* (43) tag_list ::= LB termlist */
   -3,  /* (44) tag_list ::= tag_list OR term */
   -3,  /* (45) tag_list ::= tag_list OR STOPWORD */
   -3,  /* (46) tag_list ::= tag_list OR affix */
   -3,  /* (47) tag_list ::= tag_list OR termlist */
   -2,  /* (48) tag_list ::= tag_list RB */
   -3,  /* (49) expr ::= modifier COLON numeric_range */
   -4,  /* (50) numeric_range ::= LSQB num num RSQB */
   -3,  /* (51) expr ::= modifier COLON geo_filter */
   -6,  /* (52) geo_filter ::= LSQB num num num TERM RSQB */
   -1,  /* (53) num ::= NUMBER */
   -2,  /* (54) num ::= LP num */
   -2,  /* (55) num ::= MINUS num */
   -1,  /* (56) term ::= TERM */
   -1,  /* (57) term ::= NUMBER */
};

static void yy_accept(yyParser*);  /* Forward Declaration */

/*
** Perform a reduce action and the shift that must immediately
** follow the reduce.
**
** The yyLookahead and yyLookaheadToken parameters provide reduce actions
** access to the lookahead token (if any).  The yyLookahead will be YYNOCODE
** if the lookahead token has already been consumed.  As this procedure is
** only called from one place, optimizing compilers will in-line it, which
** means that the extra parameters have no performance impact.
*/
static YYACTIONTYPE yy_reduce(
  yyParser *yypParser,         /* The parser */
  unsigned int yyruleno,       /* Number of the rule by which to reduce */
  int yyLookahead,             /* Lookahead token, or YYNOCODE if none */
  RSQueryParser_v1_TOKENTYPE yyLookaheadToken  /* Value of the lookahead token */
  RSQueryParser_v1_CTX_PDECL                   /* %extra_context */
){
  int yygoto;                     /* The next state */
  YYACTIONTYPE yyact;             /* The next action */
  yyStackEntry *yymsp;            /* The top of the parser's stack */
  int yysize;                     /* Amount to pop the stack */
  RSQueryParser_v1_ARG_FETCH
  (void)yyLookahead;
  (void)yyLookaheadToken;
  yymsp = yypParser->yytos;

  switch( yyruleno ){
  /* Beginning here are the reduction cases.  A typical example
  ** follows:
  **   case 0:
  **  #line <lineno> <grammarfile>
  **     { ... }           // User supplied code
  **  #line <lineno> <thisfile>
  **     break;
  */
/********** Begin reduce actions **********************************************/
        YYMINORTYPE yylhsminor;
      case 0: /* query ::= expr */
{ 
 /* If the root is a negative node, we intersect it with a wildcard node */
 
    ctx->root = yymsp[0].minor.yy75;
 
}
        break;
      case 1: /* query ::= */
{
    ctx->root = NULL;
}
        break;
      case 2: /* query ::= STAR */
{
    ctx->root = NewWildcardNode();
}
        break;
      case 3: /* expr ::= expr expr */
{
    int rv = one_not_null(yymsp[-1].minor.yy75, yymsp[0].minor.yy75, (void**)&yylhsminor.yy75);
    if (rv == NODENN_BOTH_INVALID) {
        yylhsminor.yy75 = NULL;
    } else if (rv == NODENN_ONE_NULL) {
        // Nothing- `out` is already assigned
    } else {
        if (yymsp[-1].minor.yy75 && yymsp[-1].minor.yy75->type == QN_PHRASE && yymsp[-1].minor.yy75->pn.exact == 0 && 
            yymsp[-1].minor.yy75->opts.fieldMask == RS_FIELDMASK_ALL ) {
            yylhsminor.yy75 = yymsp[-1].minor.yy75;
        } else {     
            yylhsminor.yy75 = NewPhraseNode(0);
            QueryNode_AddChild(yylhsminor.yy75, yymsp[-1].minor.yy75);
        }
        QueryNode_AddChild(yylhsminor.yy75, yymsp[0].minor.yy75);
    }
}
  yymsp[-1].minor.yy75 = yylhsminor.yy75;
        break;
      case 4: /* expr ::= union */
{
    yylhsminor.yy75 = yymsp[0].minor.yy75;
}
  yymsp[0].minor.yy75 = yylhsminor.yy75;
        break;
      case 5: /* union ::= expr OR expr */
{
    int rv = one_not_null(yymsp[-2].minor.yy75, yymsp[0].minor.yy75, (void**)&yylhsminor.yy75);
    if (rv == NODENN_BOTH_INVALID) {
        yylhsminor.yy75 = NULL;
    } else if (rv == NODENN_ONE_NULL) {
        // Nothing- already assigned
    } else {
        if (yymsp[-2].minor.yy75->type == QN_UNION && yymsp[-2].minor.yy75->opts.fieldMask == RS_FIELDMASK_ALL) {
            yylhsminor.yy75 = yymsp[-2].minor.yy75;
        } else {
            yylhsminor.yy75 = NewUnionNode();
            QueryNode_AddChild(yylhsminor.yy75, yymsp[-2].minor.yy75);
            yylhsminor.yy75->opts.fieldMask |= yymsp[-2].minor.yy75->opts.fieldMask;
        }

        // Handle yymsp[0].minor.yy75
        QueryNode_AddChild(yylhsminor.yy75, yymsp[0].minor.yy75);
        yylhsminor.yy75->opts.fieldMask |= yymsp[0].minor.yy75->opts.fieldMask;
        QueryNode_SetFieldMask(yylhsminor.yy75, yylhsminor.yy75->opts.fieldMask);
    }
    
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 6: /* union ::= union OR expr */
{
    yylhsminor.yy75 = yymsp[-2].minor.yy75;
    if (yymsp[0].minor.yy75) {
        QueryNode_AddChild(yylhsminor.yy75, yymsp[0].minor.yy75);
        yylhsminor.yy75->opts.fieldMask |= yymsp[0].minor.yy75->opts.fieldMask;
        QueryNode_SetFieldMask(yymsp[0].minor.yy75, yylhsminor.yy75->opts.fieldMask);
    }
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 7: /* expr ::= modifier COLON expr */
{
    if (yymsp[0].minor.yy75 == NULL) {
        yylhsminor.yy75 = NULL;
    } else {
        if (ctx->sctx->spec) {
            QueryNode_SetFieldMask(yymsp[0].minor.yy75, IndexSpec_GetFieldBit(ctx->sctx->spec, yymsp[-2].minor.yy0.s, yymsp[-2].minor.yy0.len));
        }
        yylhsminor.yy75 = yymsp[0].minor.yy75; 
    }
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 8: /* expr ::= modifierlist COLON expr */
{
    
    if (yymsp[0].minor.yy75 == NULL) {
        for (size_t i = 0; i < Vector_Size(yymsp[-2].minor.yy42); i++) {
          char *s;
          Vector_Get(yymsp[-2].minor.yy42, i, &s);
          rm_free(s);
        }
        Vector_Free(yymsp[-2].minor.yy42);
        yylhsminor.yy75 = NULL;
    } else {
        //yymsp[0].minor.yy75->opts.fieldMask = 0;
        t_fieldMask mask = 0; 
        for (int i = 0; i < Vector_Size(yymsp[-2].minor.yy42); i++) {
            char *p;
            Vector_Get(yymsp[-2].minor.yy42, i, &p);
            if (ctx->sctx->spec) {
              mask |= IndexSpec_GetFieldBit(ctx->sctx->spec, p, strlen(p));
            }
            rm_free(p);
        }
        Vector_Free(yymsp[-2].minor.yy42);
        QueryNode_SetFieldMask(yymsp[0].minor.yy75, mask);
        yylhsminor.yy75=yymsp[0].minor.yy75;
    }
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 9: /* expr ::= LP expr RP */
{
    yymsp[-2].minor.yy75 = yymsp[-1].minor.yy75;
}
        break;
      case 10: /* attribute ::= ATTRIBUTE COLON term */
{
    
    yylhsminor.yy87 = (QueryAttribute){ .name = yymsp[-2].minor.yy0.s, .namelen = yymsp[-2].minor.yy0.len, .value = rm_strndup(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len), .vallen = yymsp[0].minor.yy0.len };
}
  yymsp[-2].minor.yy87 = yylhsminor.yy87;
        break;
      case 11: /* attribute_list ::= attribute */
{
    yylhsminor.yy1 = array_new(QueryAttribute, 2);
    yylhsminor.yy1 = array_append(yylhsminor.yy1, yymsp[0].minor.yy87);
}
  yymsp[0].minor.yy1 = yylhsminor.yy1;
        break;
      case 12: /* attribute_list ::= attribute_list SEMICOLON attribute */
{
    yylhsminor.yy1 = array_append(yymsp[-2].minor.yy1, yymsp[0].minor.yy87);
}
  yymsp[-2].minor.yy1 = yylhsminor.yy1;
        break;
      case 13: /* attribute_list ::= attribute_list SEMICOLON */
{
    yylhsminor.yy1 = yymsp[-1].minor.yy1;
}
  yymsp[-1].minor.yy1 = yylhsminor.yy1;
        break;
      case 14: /* attribute_list ::= */
{
    yymsp[1].minor.yy1 = NULL;
}
        break;
      case 15: /* expr ::= expr ARROW LB attribute_list RB */
{

    if (yymsp[-4].minor.yy75 && yymsp[-1].minor.yy1) {
        QueryNode_ApplyAttributes(yymsp[-4].minor.yy75, yymsp[-1].minor.yy1, array_len(yymsp[-1].minor.yy1), ctx->status);
    }
    array_free_ex(yymsp[-1].minor.yy1, rm_free((char*)((QueryAttribute*)ptr )->value));
    yylhsminor.yy75 = yymsp[-4].minor.yy75;
}
  yymsp[-4].minor.yy75 = yylhsminor.yy75;
        break;
      case 16: /* expr ::= QUOTE termlist QUOTE */
{
    yymsp[-1].minor.yy75->pn.exact =1;
    yymsp[-1].minor.yy75->opts.flags |= QueryNode_Verbatim;

    yymsp[-2].minor.yy75 = yymsp[-1].minor.yy75;
}
        break;
      case 17: /* expr ::= QUOTE term QUOTE */
{
    yymsp[-2].minor.yy75 = NewTokenNode(ctx, strdupcase(yymsp[-1].minor.yy0.s, yymsp[-1].minor.yy0.len), -1);
    yymsp[-2].minor.yy75->opts.flags |= QueryNode_Verbatim;
    
}
        break;
      case 18: /* expr ::= term */
{
   yylhsminor.yy75 = NewTokenNode(ctx, strdupcase(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len), -1);
}
  yymsp[0].minor.yy75 = yylhsminor.yy75;
        break;
      case 19: /* expr ::= affix */
{
    yylhsminor.yy75= yymsp[0].minor.yy75;
}
  yymsp[0].minor.yy75 = yylhsminor.yy75;
        break;
      case 20: /* expr ::= termlist */
{
        yylhsminor.yy75 = yymsp[0].minor.yy75;
}
  yymsp[0].minor.yy75 = yylhsminor.yy75;
        break;
      case 21: /* expr ::= STOPWORD */
{
    yymsp[0].minor.yy75 = NULL;
}
        break;
      case 22: /* termlist ::= term term */
{
    yylhsminor.yy75 = NewPhraseNode(0);
    QueryNode_AddChild(yylhsminor.yy75, NewTokenNode(ctx, strdupcase(yymsp[-1].minor.yy0.s, yymsp[-1].minor.yy0.len), -1));
    QueryNode_AddChild(yylhsminor.yy75, NewTokenNode(ctx, strdupcase(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len), -1));
}
  yymsp[-1].minor.yy75 = yylhsminor.yy75;
        break;
      case 23: /* termlist ::= termlist term */
{
    yylhsminor.yy75 = yymsp[-1].minor.yy75;
    QueryNode_AddChild(yylhsminor.yy75, NewTokenNode(ctx, strdupcase(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len), -1));
}
  yymsp[-1].minor.yy75 = yylhsminor.yy75;
        break;
      case 24: /* termlist ::= termlist STOPWORD */
      case 48: /* tag_list ::= tag_list RB */ yytestcase(yyruleno==48);
{
    yylhsminor.yy75 = yymsp[-1].minor.yy75;
}
  yymsp[-1].minor.yy75 = yylhsminor.yy75;
        break;
      case 25: /* expr ::= MINUS expr */
{ 
    if (yymsp[0].minor.yy75) {
        yymsp[-1].minor.yy75 = NewNotNode(yymsp[0].minor.yy75);
    } else {
        yymsp[-1].minor.yy75 = NULL;
    }
}
        break;
      case 26: /* expr ::= TILDE expr */
{ 
    if (yymsp[0].minor.yy75) {
        yymsp[-1].minor.yy75 = NewOptionalNode(yymsp[0].minor.yy75);
    } else {
        yymsp[-1].minor.yy75 = NULL;
    }
}
        break;
      case 27: /* affix ::= PREFIX */
{
    yylhsminor.yy75 = NewPrefixNode_WithParams(ctx, &yymsp[0].minor.yy0, true, false);
}
  yymsp[0].minor.yy75 = yylhsminor.yy75;
        break;
      case 28: /* affix ::= SUFFIX */
{
    yylhsminor.yy75 = NewPrefixNode_WithParams(ctx, &yymsp[0].minor.yy0, false, true);
}
  yymsp[0].minor.yy75 = yylhsminor.yy75;
        break;
      case 29: /* affix ::= CONTAINS */
{
    yylhsminor.yy75 = NewPrefixNode_WithParams(ctx, &yymsp[0].minor.yy0, true, true);
}
  yymsp[0].minor.yy75 = yylhsminor.yy75;
        break;
      case 30: /* expr ::= PERCENT term PERCENT */
      case 33: /* expr ::= PERCENT STOPWORD PERCENT */ yytestcase(yyruleno==33);
{
    yymsp[-2].minor.yy75 = NewFuzzyNode_WithParams(ctx, &yymsp[-1].minor.yy0, 1);
}
        break;
      case 31: /* expr ::= PERCENT PERCENT term PERCENT PERCENT */
      case 34: /* expr ::= PERCENT PERCENT STOPWORD PERCENT PERCENT */ yytestcase(yyruleno==34);
{
    yymsp[-4].minor.yy75 = NewFuzzyNode_WithParams(ctx, &yymsp[-2].minor.yy0, 2);
}
        break;
      case 32: /* expr ::= PERCENT PERCENT PERCENT term PERCENT PERCENT PERCENT */
      case 35: /* expr ::= PERCENT PERCENT PERCENT STOPWORD PERCENT PERCENT PERCENT */ yytestcase(yyruleno==35);
{
    yymsp[-6].minor.yy75 = NewFuzzyNode_WithParams(ctx, &yymsp[-3].minor.yy0, 3);
}
        break;
      case 36: /* modifier ::= MODIFIER */
{
    yymsp[0].minor.yy0.len = unescapen((char*)yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len);
    yylhsminor.yy0 = yymsp[0].minor.yy0;
 }
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      case 37: /* modifierlist ::= modifier OR term */
{
    yylhsminor.yy42 = NewVector(char *, 2);
    char *s = rm_strndup(yymsp[-2].minor.yy0.s, yymsp[-2].minor.yy0.len);
    Vector_Push(yylhsminor.yy42, s);
    s = rm_strndup(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len);
    Vector_Push(yylhsminor.yy42, s);
}
  yymsp[-2].minor.yy42 = yylhsminor.yy42;
        break;
      case 38: /* modifierlist ::= modifierlist OR term */
{
    char *s = rm_strndup(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len);
    Vector_Push(yymsp[-2].minor.yy42, s);
    yylhsminor.yy42 = yymsp[-2].minor.yy42;
}
  yymsp[-2].minor.yy42 = yylhsminor.yy42;
        break;
      case 39: /* expr ::= modifier COLON tag_list */
{
    if (!yymsp[0].minor.yy75) {
        yylhsminor.yy75= NULL;
    } else {
        // Tag field names must be case sensitive, we we can't do strdupcase
        char *s = rm_strndup(yymsp[-2].minor.yy0.s, yymsp[-2].minor.yy0.len);
        size_t slen = unescapen((char*)s, yymsp[-2].minor.yy0.len);

        yylhsminor.yy75 = NewTagNode(s, slen);
        QueryNode_AddChildren(yylhsminor.yy75, yymsp[0].minor.yy75->children, QueryNode_NumChildren(yymsp[0].minor.yy75));
        
        // Set the children count on yymsp[0].minor.yy75 to 0 so they won't get recursively free'd
        QueryNode_ClearChildren(yymsp[0].minor.yy75, 0);
        QueryNode_Free(yymsp[0].minor.yy75);
    }
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 40: /* tag_list ::= LB term */
      case 41: /* tag_list ::= LB STOPWORD */ yytestcase(yyruleno==41);
{
    yymsp[-1].minor.yy75 = NewPhraseNode(0);
    QueryNode_AddChild(yymsp[-1].minor.yy75, NewTokenNode(ctx, rm_strndup(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len), -1));
}
        break;
      case 42: /* tag_list ::= LB affix */
      case 43: /* tag_list ::= LB termlist */ yytestcase(yyruleno==43);
{
    yymsp[-1].minor.yy75 = NewPhraseNode(0);
    QueryNode_AddChild(yymsp[-1].minor.yy75, yymsp[0].minor.yy75);
}
        break;
      case 44: /* tag_list ::= tag_list OR term */
      case 45: /* tag_list ::= tag_list OR STOPWORD */ yytestcase(yyruleno==45);
{
    QueryNode_AddChild(yymsp[-2].minor.yy75, NewTokenNode(ctx, rm_strndup(yymsp[0].minor.yy0.s, yymsp[0].minor.yy0.len), -1));
    yylhsminor.yy75 = yymsp[-2].minor.yy75;
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 46: /* tag_list ::= tag_list OR affix */
      case 47: /* tag_list ::= tag_list OR termlist */ yytestcase(yyruleno==47);
{
    QueryNode_AddChild(yymsp[-2].minor.yy75, yymsp[0].minor.yy75);
    yylhsminor.yy75 = yymsp[-2].minor.yy75;
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 49: /* expr ::= modifier COLON numeric_range */
{
    // we keep the capitalization as is
    yymsp[0].minor.yy62->nf->fieldName = rm_strndup(yymsp[-2].minor.yy0.s, yymsp[-2].minor.yy0.len);
    yylhsminor.yy75 = NewNumericNode(yymsp[0].minor.yy62);
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 50: /* numeric_range ::= LSQB num num RSQB */
{
  yymsp[-3].minor.yy62 = NewQueryParam(QP_NUMERIC_FILTER);
  yymsp[-3].minor.yy62->nf = NewNumericFilter(yymsp[-2].minor.yy47.num, yymsp[-1].minor.yy47.num, yymsp[-2].minor.yy47.inclusive, yymsp[-1].minor.yy47.inclusive);
}
        break;
      case 51: /* expr ::= modifier COLON geo_filter */
{
    // we keep the capitalization as is
    yymsp[0].minor.yy62->gf->property = rm_strndup(yymsp[-2].minor.yy0.s, yymsp[-2].minor.yy0.len);
    yylhsminor.yy75 = NewGeofilterNode(yymsp[0].minor.yy62);
}
  yymsp[-2].minor.yy75 = yylhsminor.yy75;
        break;
      case 52: /* geo_filter ::= LSQB num num num TERM RSQB */
{
    yymsp[-5].minor.yy62 = NewQueryParam(QP_GEO_FILTER);
    yymsp[-5].minor.yy62->gf = NewGeoFilter(yymsp[-4].minor.yy47.num, yymsp[-3].minor.yy47.num, yymsp[-2].minor.yy47.num, yymsp[-1].minor.yy0.s, yymsp[-1].minor.yy0.len);
    GeoFilter_Validate(yymsp[-5].minor.yy62->gf, ctx->status);
}
        break;
      case 53: /* num ::= NUMBER */
{
    yylhsminor.yy47.num = yymsp[0].minor.yy0.numval;
    yylhsminor.yy47.inclusive = 1;
}
  yymsp[0].minor.yy47 = yylhsminor.yy47;
        break;
      case 54: /* num ::= LP num */
{
    yymsp[-1].minor.yy47=yymsp[0].minor.yy47;
    yymsp[-1].minor.yy47.inclusive = 0;
}
        break;
      case 55: /* num ::= MINUS num */
{
    yymsp[0].minor.yy47.num = -yymsp[0].minor.yy47.num;
    yymsp[-1].minor.yy47 = yymsp[0].minor.yy47;
}
        break;
      case 56: /* term ::= TERM */
      case 57: /* term ::= NUMBER */ yytestcase(yyruleno==57);
{
    yylhsminor.yy0 = yymsp[0].minor.yy0; 
}
  yymsp[0].minor.yy0 = yylhsminor.yy0;
        break;
      default:
        break;
/********** End reduce actions ************************************************/
  };
  assert( yyruleno<sizeof(yyRuleInfoLhs)/sizeof(yyRuleInfoLhs[0]) );
  yygoto = yyRuleInfoLhs[yyruleno];
  yysize = yyRuleInfoNRhs[yyruleno];
  yyact = yy_find_reduce_action(yymsp[yysize].stateno,(YYCODETYPE)yygoto);

  /* There are no SHIFTREDUCE actions on nonterminals because the table
  ** generator has simplified them to pure REDUCE actions. */
  assert( !(yyact>YY_MAX_SHIFT && yyact<=YY_MAX_SHIFTREDUCE) );

  /* It is not possible for a REDUCE to be followed by an error */
  assert( yyact!=YY_ERROR_ACTION );

  yymsp += yysize+1;
  yypParser->yytos = yymsp;
  yymsp->stateno = (YYACTIONTYPE)yyact;
  yymsp->major = (YYCODETYPE)yygoto;
  yyTraceShift(yypParser, yyact, "... then shift");
  return yyact;
}

/*
** The following code executes when the parse fails
*/
#ifndef YYNOERRORRECOVERY
static void yy_parse_failed(
  yyParser *yypParser           /* The parser */
){
  RSQueryParser_v1_ARG_FETCH
  RSQueryParser_v1_CTX_FETCH
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
  }
#endif
  while( yypParser->yytos>yypParser->yystack ) yy_pop_parser_stack(yypParser);
  /* Here code is inserted which will be executed whenever the
  ** parser fails */
/************ Begin %parse_failure code ***************************************/
/************ End %parse_failure code *****************************************/
  RSQueryParser_v1_ARG_STORE /* Suppress warning about unused %extra_argument variable */
  RSQueryParser_v1_CTX_STORE
}
#endif /* YYNOERRORRECOVERY */

/*
** The following code executes when a syntax error first occurs.
*/
static void yy_syntax_error(
  yyParser *yypParser,           /* The parser */
  int yymajor,                   /* The major type of the error token */
  RSQueryParser_v1_TOKENTYPE yyminor         /* The minor type of the error token */
){
  RSQueryParser_v1_ARG_FETCH
  RSQueryParser_v1_CTX_FETCH
#define TOKEN yyminor
/************ Begin %syntax_error code ****************************************/
  
    QueryError_SetErrorFmt(ctx->status, QUERY_ESYNTAX,
        "Syntax error at offset %d near %.*s",
        TOKEN.pos, TOKEN.len, TOKEN.s);
/************ End %syntax_error code ******************************************/
  RSQueryParser_v1_ARG_STORE /* Suppress warning about unused %extra_argument variable */
  RSQueryParser_v1_CTX_STORE
}

/*
** The following is executed when the parser accepts
*/
static void yy_accept(
  yyParser *yypParser           /* The parser */
){
  RSQueryParser_v1_ARG_FETCH
  RSQueryParser_v1_CTX_FETCH
#ifndef NDEBUG
  if( yyTraceFILE ){
    fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
  }
#endif
#ifndef YYNOERRORRECOVERY
  yypParser->yyerrcnt = -1;
#endif
  assert( yypParser->yytos==yypParser->yystack );
  /* Here code is inserted which will be executed whenever the
  ** parser accepts */
/*********** Begin %parse_accept code *****************************************/
/*********** End %parse_accept code *******************************************/
  RSQueryParser_v1_ARG_STORE /* Suppress warning about unused %extra_argument variable */
  RSQueryParser_v1_CTX_STORE
}

/* The main parser program.
** The first argument is a pointer to a structure obtained from
** "RSQueryParser_v1_Alloc" which describes the current state of the parser.
** The second argument is the major token number.  The third is
** the minor token.  The fourth optional argument is whatever the
** user wants (and specified in the grammar) and is available for
** use by the action routines.
**
** Inputs:
** <ul>
** <li> A pointer to the parser (an opaque structure.)
** <li> The major token number.
** <li> The minor token number.
** <li> An option argument of a grammar-specified type.
** </ul>
**
** Outputs:
** None.
*/
void RSQueryParser_v1_(
  void *yyp,                   /* The parser */
  int yymajor,                 /* The major token code number */
  RSQueryParser_v1_TOKENTYPE yyminor       /* The value for the token */
  RSQueryParser_v1_ARG_PDECL               /* Optional %extra_argument parameter */
){
  YYMINORTYPE yyminorunion;
  YYACTIONTYPE yyact;   /* The parser action. */
#if !defined(YYERRORSYMBOL) && !defined(YYNOERRORRECOVERY)
  int yyendofinput;     /* True if we are at the end of input */
#endif
#ifdef YYERRORSYMBOL
  int yyerrorhit = 0;   /* True if yymajor has invoked an error */
#endif
  yyParser *yypParser = (yyParser*)yyp;  /* The parser */
  RSQueryParser_v1_CTX_FETCH
  RSQueryParser_v1_ARG_STORE

  assert( yypParser->yytos!=0 );
#if !defined(YYERRORSYMBOL) && !defined(YYNOERRORRECOVERY)
  yyendofinput = (yymajor==0);
#endif

  yyact = yypParser->yytos->stateno;
#ifndef NDEBUG
  if( yyTraceFILE ){
    if( yyact < YY_MIN_REDUCE ){
      fprintf(yyTraceFILE,"%sInput '%s' in state %d\n",
              yyTracePrompt,yyTokenName[yymajor],yyact);
    }else{
      fprintf(yyTraceFILE,"%sInput '%s' with pending reduce %d\n",
              yyTracePrompt,yyTokenName[yymajor],yyact-YY_MIN_REDUCE);
    }
  }
#endif

  while(1){ /* Exit by "break" */
    assert( yypParser->yytos>=yypParser->yystack );
    assert( yyact==yypParser->yytos->stateno );
    yyact = yy_find_shift_action((YYCODETYPE)yymajor,yyact);
    if( yyact >= YY_MIN_REDUCE ){
      unsigned int yyruleno = yyact - YY_MIN_REDUCE; /* Reduce by this rule */
#ifndef NDEBUG
      assert( yyruleno<(int)(sizeof(yyRuleName)/sizeof(yyRuleName[0])) );
      if( yyTraceFILE ){
        int yysize = yyRuleInfoNRhs[yyruleno];
        if( yysize ){
          fprintf(yyTraceFILE, "%sReduce %d [%s]%s, pop back to state %d.\n",
            yyTracePrompt,
            yyruleno, yyRuleName[yyruleno],
            yyruleno<YYNRULE_WITH_ACTION ? "" : " without external action",
            yypParser->yytos[yysize].stateno);
        }else{
          fprintf(yyTraceFILE, "%sReduce %d [%s]%s.\n",
            yyTracePrompt, yyruleno, yyRuleName[yyruleno],
            yyruleno<YYNRULE_WITH_ACTION ? "" : " without external action");
        }
      }
#endif /* NDEBUG */

      /* Check that the stack is large enough to grow by a single entry
      ** if the RHS of the rule is empty.  This ensures that there is room
      ** enough on the stack to push the LHS value */
      if( yyRuleInfoNRhs[yyruleno]==0 ){
#ifdef YYTRACKMAXSTACKDEPTH
        if( (int)(yypParser->yytos - yypParser->yystack)>yypParser->yyhwm ){
          yypParser->yyhwm++;
          assert( yypParser->yyhwm ==
                  (int)(yypParser->yytos - yypParser->yystack));
        }
#endif
#if YYSTACKDEPTH>0 
        if( yypParser->yytos>=yypParser->yystackEnd ){
          yyStackOverflow(yypParser);
          break;
        }
#else
        if( yypParser->yytos>=&yypParser->yystack[yypParser->yystksz-1] ){
          if( yyGrowStack(yypParser) ){
            yyStackOverflow(yypParser);
            break;
          }
        }
#endif
      }
      yyact = yy_reduce(yypParser,yyruleno,yymajor,yyminor RSQueryParser_v1_CTX_PARAM);
    }else if( yyact <= YY_MAX_SHIFTREDUCE ){
      yy_shift(yypParser,yyact,(YYCODETYPE)yymajor,yyminor);
#ifndef YYNOERRORRECOVERY
      yypParser->yyerrcnt--;
#endif
      break;
    }else if( yyact==YY_ACCEPT_ACTION ){
      yypParser->yytos--;
      yy_accept(yypParser);
      return;
    }else{
      assert( yyact == YY_ERROR_ACTION );
      yyminorunion.yy0 = yyminor;
#ifdef YYERRORSYMBOL
      int yymx;
#endif
#ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
      }
#endif
#ifdef YYERRORSYMBOL
      /* A syntax error has occurred.
      ** The response to an error depends upon whether or not the
      ** grammar defines an error token "ERROR".  
      **
      ** This is what we do if the grammar does define ERROR:
      **
      **  * Call the %syntax_error function.
      **
      **  * Begin popping the stack until we enter a state where
      **    it is legal to shift the error symbol, then shift
      **    the error symbol.
      **
      **  * Set the error count to three.
      **
      **  * Begin accepting and shifting new tokens.  No new error
      **    processing will occur until three tokens have been
      **    shifted successfully.
      **
      */
      if( yypParser->yyerrcnt<0 ){
        yy_syntax_error(yypParser,yymajor,yyminor);
      }
      yymx = yypParser->yytos->major;
      if( yymx==YYERRORSYMBOL || yyerrorhit ){
#ifndef NDEBUG
        if( yyTraceFILE ){
          fprintf(yyTraceFILE,"%sDiscard input token %s\n",
             yyTracePrompt,yyTokenName[yymajor]);
        }
#endif
        yy_destructor(yypParser, (YYCODETYPE)yymajor, &yyminorunion);
        yymajor = YYNOCODE;
      }else{
        while( yypParser->yytos > yypParser->yystack ){
          yyact = yy_find_reduce_action(yypParser->yytos->stateno,
                                        YYERRORSYMBOL);
          if( yyact<=YY_MAX_SHIFTREDUCE ) break;
          yy_pop_parser_stack(yypParser);
        }
        if( yypParser->yytos <= yypParser->yystack || yymajor==0 ){
          yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
          yy_parse_failed(yypParser);
#ifndef YYNOERRORRECOVERY
          yypParser->yyerrcnt = -1;
#endif
          yymajor = YYNOCODE;
        }else if( yymx!=YYERRORSYMBOL ){
          yy_shift(yypParser,yyact,YYERRORSYMBOL,yyminor);
        }
      }
      yypParser->yyerrcnt = 3;
      yyerrorhit = 1;
      if( yymajor==YYNOCODE ) break;
      yyact = yypParser->yytos->stateno;
#elif defined(YYNOERRORRECOVERY)
      /* If the YYNOERRORRECOVERY macro is defined, then do not attempt to
      ** do any kind of error recovery.  Instead, simply invoke the syntax
      ** error routine and continue going as if nothing had happened.
      **
      ** Applications can set this macro (for example inside %include) if
      ** they intend to abandon the parse upon the first syntax error seen.
      */
      yy_syntax_error(yypParser,yymajor, yyminor);
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      break;
#else  /* YYERRORSYMBOL is not defined */
      /* This is what we do if the grammar does not define ERROR:
      **
      **  * Report an error message, and throw away the input token.
      **
      **  * If the input token is $, then fail the parse.
      **
      ** As before, subsequent error messages are suppressed until
      ** three input tokens have been successfully shifted.
      */
      if( yypParser->yyerrcnt<=0 ){
        yy_syntax_error(yypParser,yymajor, yyminor);
      }
      yypParser->yyerrcnt = 3;
      yy_destructor(yypParser,(YYCODETYPE)yymajor,&yyminorunion);
      if( yyendofinput ){
        yy_parse_failed(yypParser);
#ifndef YYNOERRORRECOVERY
        yypParser->yyerrcnt = -1;
#endif
      }
      break;
#endif
    }
  }
#ifndef NDEBUG
  if( yyTraceFILE ){
    yyStackEntry *i;
    char cDiv = '[';
    fprintf(yyTraceFILE,"%sReturn. Stack=",yyTracePrompt);
    for(i=&yypParser->yystack[1]; i<=yypParser->yytos; i++){
      fprintf(yyTraceFILE,"%c%s", cDiv, yyTokenName[i->major]);
      cDiv = ' ';
    }
    fprintf(yyTraceFILE,"]\n");
  }
#endif
  return;
}

/*
** Return the fallback token corresponding to canonical token iToken, or
** 0 if iToken has no fallback.
*/
int RSQueryParser_v1_Fallback(int iToken){
#ifdef YYFALLBACK
  assert( iToken<(int)(sizeof(yyFallback)/sizeof(yyFallback[0])) );
  return yyFallback[iToken];
#else
  (void)iToken;
  return 0;
#endif
}
